// Class under testing
import IOSWorker from './iossyncworker';

// Dependencies
import {} from 'jasmine';

import { fakeAsync, tick } from '@angular/core/testing';

describe('IOSWorker', () => {
	let worker = new IOSWorker();

	it('should define a worker', () => {
		expect(worker instanceof IOSWorker).toBe(true);
	});

	describe('.sendMessage()', () => {
		it('should call listener method with right params', () => {
			// Prepare
			const spy = jasmine.createSpy('listener');
			worker['listener'] = spy;

			// Execute
			worker.sendMessage('fake message');

			// Expect
			expect(spy).toHaveBeenCalledWith({ data: 'fake message' });
		});
	});

	describe('.postMessage()', () => {
		beforeAll(() => {
			spyOn(worker, 'sendMessage').and.stub();
			spyOn(worker['syncWorker'], 'setLoggerLevel').and.stub();
			spyOn(worker['syncWorker'].endpoints, 'setup').and.stub();
			spyOn(worker['syncWorker'].endpoints, 'snapshotUpdate').and.stub();
			spyOn(worker['syncWorker'].endpoints, 'abortAll').and.stub();
		});

		it('should call sendMessage with right parameters when message is "ENDPOINTS_UPDATE"', () => {
			// Execute
			worker.postMessage({ message: 'ENDPOINTS_UPDATE' });

			// Expect
			expect(worker.sendMessage).toHaveBeenCalledWith({ message: 'LOG', payload: 'this.syncWorker: Received ENDPOINTS_UPDATE request' });
		});

		it('should call endpoints.setup with right parameters when message is "ENDPOINTS_UPDATE"', () => {
			// Execute
			worker.postMessage({ message: 'ENDPOINTS_UPDATE', payload: 'fake payload', metadata: 'fake metadata' });

			// Expect
			expect(worker['syncWorker'].endpoints.setup).toHaveBeenCalledWith('fake payload', 'fake metadata');
		});

		it('should call sendMessage with right parameters when message is "SNAPSHOT_UPDATE"', () => {
			// Execute
			worker.postMessage({ message: 'SNAPSHOT_UPDATE' });

			// Expect
			expect(worker.sendMessage).toHaveBeenCalledWith({ message: 'LOG', payload: 'this.syncWorker: Received SNAPSHOT_UPDATE request' });
		});

		it('should call endpoints.setup with right parameters when message is "SNAPSHOT_UPDATE"', () => {
			// Execute
			worker.postMessage({ message: 'SNAPSHOT_UPDATE', payload: 'fake payload', metadata: 'fake metadata' });

			// Expect
			expect(worker['syncWorker'].endpoints.snapshotUpdate).toHaveBeenCalledWith('fake payload', 'fake metadata');
		});

		it('should call sendMessage with right parameters when message is "REQUEST_CANCEL"', () => {
			// Execute
			worker.postMessage({ message: 'REQUEST_CANCEL' });

			// Expect
			expect(worker.sendMessage).toHaveBeenCalledWith({ message: 'LOG', payload: 'this.syncWorker: Received REQUEST_CANCEL request' });
		});

		it('should call endpoints.setup with right parameters when message is "REQUEST_CANCEL"', () => {
			// Execute
			worker.postMessage({ message: 'REQUEST_CANCEL' });

			// Expect
			expect(worker['syncWorker'].endpoints.abortAll).toHaveBeenCalled();
		});

		it('should set httpTimeout when message is "SET_TIMEOUT"', () => {
			// Execute
			worker.postMessage({ message: 'SET_TIMEOUT', timeout: 30 });

			// Expect
			expect(worker['httpTimeout']).toBe(30);
		});

		it('should not call sendMessage when message is not recognized', () => {
			// Prepare
			const spyCalls = (worker.sendMessage as jasmine.Spy).calls.count();

			// Execute
			worker.postMessage({ message: 'invaid_message' });

			// Expect
			expect((worker.sendMessage as jasmine.Spy).calls.count()).toBe(spyCalls);
		});
	});

	describe('.addEventListener()', () => {
		it('should set the listener property', () => {
			// Prepare
			const newCallback = () => {};

			// Execute
			worker.addEventListener('fake message', newCallback);

			// Prepare
			expect(worker['listener']).toBe(newCallback);
		});
	});

	describe('.terminate()', () => {
		it('should call removeAllListeners method', () => {
			// Prepare
			worker['removeAllListeners'] = jasmine.createSpy('removeAllListeners');

			// Execute
			worker.terminate();

			// Expect
			expect(worker['removeAllListeners']).toHaveBeenCalled();
		});
	});

	describe('.addParameterToURL()', () => {
		it('should add the given parameter to the URL as the first parameter when the URL has not any parameter', () => {
			// Execute
			const result = worker.addParameterToURL('search', 'id', 5);

			// Expect
			expect(result).toBe('search?id=5');
		});

		it('should append the given parameter to the URL when the URL has some parameter', () => {
			// Execute
			const result = worker.addParameterToURL('search?id=5', 'name', 'testing');

			// Expect
			expect(result).toBe('search?id=5&name=testing');
		});
	});

	describe('.loadRemote()', () => {
		let xhr = window.XMLHttpRequest,
			xhrSpy;

		beforeAll(() => {
			xhrSpy = jasmine.createSpyObj('xhrObj', [
				'abort',
				'addEventListener',
				'open',
				'send',
				'setRequestHeader'
			]);
			spyOn(window, 'XMLHttpRequest').and.returnValue(xhrSpy);
			spyOn(worker['syncWorker'].logger, 'log').and.stub();
		});

		afterAll(() => {
			window.XMLHttpRequest = xhr;
		});

		it('should call syncWorker.logger.log with right parameters', () => {
			// Execute
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});

			// Expect
			expect(worker['syncWorker'].logger.log).toHaveBeenCalledWith('loadRemote, class: fake class, url: fake url');
		});

		it('should call abort method on timeout', fakeAsync(() => {
			// Prepare
			worker['httpTimeout'] = 10;

			// Execute
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});
			tick(10);

			// Expect
			expect(xhrSpy.abort).toHaveBeenCalled();
		}));

		it('should set isNoResponse property to true when there is an error and the ready state is bigger than 3', () => {
			// Prepare
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});
			xhrSpy['readyState'] = 4;

			// Execute
			xhrSpy.onerror();

			// Expect
			expect(xhrSpy.isNoResponse).toBe(true);
		});

		it('should not set isNoResponse property when there is an error and the ready state is less than 4', () => {
			// Prepare
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});
			xhrSpy['readyState'] = 3;
			xhrSpy['isNoResponse'] = undefined;

			// Execute
			xhrSpy.onerror();

			// Expect
			expect(xhrSpy.isNoResponse).not.toBeDefined();
		});

		it('should define the responseTime property on onloadend event and the readyState is bigger than 3', () => {
			// Prepare
			xhrSpy['readyState'] = 4;
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});

			// Execute
			xhrSpy.onloadend();

			// Expect
			expect(xhrSpy.responseTime).toBeDefined();
		});

		it('should not define the responseTime property on onloadend event and the readyState is less than 4', () => {
			// Prepare
			xhrSpy['readyState'] = 3;
			xhrSpy.responseTime = undefined;
			worker.loadRemote('fake class', 'fake url', () => {}, () => {}, {});

			// Execute
			xhrSpy.onloadend();

			// Expect
			expect(xhrSpy.responseTime).not.toBeDefined();
		});

		it('should call success callback on onloadend event and the status is between 2xx-3xx', () => {
			// Prepare
			const spy = jasmine.createSpy();
			xhrSpy['readyState'] = 4;
			xhrSpy['status'] = 200;
			worker.loadRemote('fake class', 'fake url', spy, () => {}, {});

			// Execute
			xhrSpy.onloadend();

			// Expect
			expect(spy).toHaveBeenCalledWith('fake class', xhrSpy);
		});

		it('should not call success or error callback on onloadend event when is canceled', () => {
			// Prepare
			const successSpy = jasmine.createSpy();
			const errorSpy = jasmine.createSpy();
			xhrSpy['readyState'] = 4;
			worker.loadRemote('fake class', 'fake url', successSpy, errorSpy, {});
			xhrSpy['canceled'] = true;

			// Execute
			xhrSpy.onloadend();

			// Expect
			expect(successSpy).not.toHaveBeenCalled();
			expect(errorSpy).not.toHaveBeenCalled();
		});
	});

	describe('.syncWorker', () => {
		describe('.logger.log()', () => {
			it('should call console.log with right parameters', () => {
				// Execute
				worker['syncWorker'].logger.log('fake location', 'fake event');

				// Expect
				expect(window.console.log).toHaveBeenCalledWith('WebWorker - fake location - fake event');
			});

			it('should call console.log without the event name', () => {
				// Execute
				worker['syncWorker'].logger.log('fake location');

				// Expect
				expect(window.console.log).toHaveBeenCalledWith('WebWorker - fake location');
			});

			it('should not call console.log when conf.log is not defined', () => {
				// Prepare
				const originalConf = worker['conf'].log;
				const spyCalls = (window.console.log as jasmine.Spy).calls.count();
				worker['conf'].log = false;

				// Execute
				worker['syncWorker'].logger.log('fake location');

				// Restore
				worker['conf'].log = originalConf;

				// Expect
				expect((window.console.log as jasmine.Spy).calls.count()).toBe(spyCalls);
			});
		});

		describe('.setLoggerLevel()', () => {
			const originalConf = worker['conf'].log;

			afterEach(() => {
				worker['conf'].log = originalConf;
			});

			it('should set conf.log to false when options are null', () => {
				worker['syncWorker'].setLoggerLevel(null);

				// Expect
				expect(worker['conf'].log).toBe(false);
			});

			it('should set conf.log to true when given devMode is true', () => {
				worker['syncWorker'].setLoggerLevel({ devMode: true });

				// Expect
				expect(worker['conf'].log).toBe(true);
			});
		});

		describe('.communication.sendDataMessage()', () => {
			it('should call sendMessage with right parameters', () => {
				// Prepare
				spyOn(worker, 'sendMessage').and.stub();

				// Execute
				worker['syncWorker'].communication.sendDataMessage('fake event', 'fake class', { name: 'fake data name' });

				// Expect
				expect(worker.sendMessage).toHaveBeenCalledWith({
					message: 'fake event',
					payload: {
						class: 'fake class',
						data: {
							name: 'fake data name'
						}
					}
				});
			});
		});

		describe('.endpoints', () => {
			describe('.setup()', () => {
				beforeEach(() => {
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker['syncWorker'].endpoints, 'fetch').and.stub();
				});

				it('should call syncWorker.logger.log with right parameters', () => {
					// Execute
					worker['syncWorker'].endpoints.setup({ class: 'fake class', url: 'test-url'}, { extra: 'fake extra' });

					// Expect
					expect(worker['syncWorker'].logger.log).toHaveBeenCalledWith('SETTING UP ENDPOINT URL: test-url');
				});

				it('should call syncWorker.endpoints.fetch with right parameters', () => {
					// Execute
					worker['syncWorker'].endpoints.setup({ class: 'fake class', url: 'test-url'}, { extra: 'fake extra' });

					// Expect
					expect(worker['syncWorker'].endpoints.fetch).toHaveBeenCalledWith('fake class', 'test-url', { extra: 'fake extra' });
				});
			});

			describe('.processXhrResponse()', () => {
				it('should return a payload with data when responseText is a valid JSON', () => {
					// Prepare
					const result = worker['syncWorker'].endpoints.processXhrResponse({
						responseText: '{ "message": "fake message" }',
						status: 200,
						responseTime: 100,
						responseURL: 'fake'
					});

					// Expect
					expect(result).toEqual({
						hash: jasmine.any(Number),
						status: 200,
						responseTime: 100,
						responseURL: 'fake',
						data: { message: 'fake message' },
						class: ''
					});
				});

				it('should return a payload with data null when responseText is not a valid JSON', () => {
					// Prepare
					const result = worker['syncWorker'].endpoints.processXhrResponse({
						responseText: '',
						status: 200,
						responseTime: 100,
						responseURL: 'fake'
					});

					// Expect
					expect(result).toEqual({
						hash: null,
						status: 200,
						responseTime: 100,
						responseURL: 'fake',
						data: null,
						class: ''
					});
				});
			});

			describe('.processXhrErrorResponse()', () => {
				it('should return a payload error with given parameters', () => {
					// Prepare
					const result = worker['syncWorker'].endpoints.processXhrErrorResponse({
						responseText: '',
						status: 500,
						responseTime: 100,
						responseURL: 'fake'
					});

					// Expect
					expect(result).toEqual({
						status: 500,
						responseTime: 100,
						responseURL: 'fake',
						data: ''
					});
				});

				it('should return a payload error with zero as status when isNoResponse is true', () => {
					// Prepare
					const result = worker['syncWorker'].endpoints.processXhrErrorResponse({
						responseText: '',
						responseTime: 100,
						responseURL: 'fake',
						isNoResponse: true
					});

					// Expect
					expect(result).toEqual({
						status: 0,
						responseTime: 100,
						responseURL: 'fake',
						data: ''
					});
				});

				it('should return a payload error with failedURL as responseURL when that URL is not defined', () => {
					// Prepare
					const result = worker['syncWorker'].endpoints.processXhrErrorResponse({
						responseText: '',
						responseTime: 100,
						failedURL: 'another',
						isNoResponse: true
					});

					// Expect
					expect(result).toEqual({
						status: 0,
						responseTime: 100,
						responseURL: 'another',
						data: ''
					});
				});
			});

			describe('.fetch()', () => {
				it('should call loadRemote with right parameters', () => {
					// Prepare
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker, 'loadRemote').and.stub();

					// Execute
					worker['syncWorker'].endpoints.fetch('fake class', 'url', { name: 'name test' });

					// Expect
					expect(worker.loadRemote).toHaveBeenCalledWith(
						'fake class',
						'url',
						worker['syncWorker'].endpoints.fetchCallback,
						worker['syncWorker'].endpoints.fetchErrorCallback,
						{ name: 'name test' }
					);
				});
			});

			describe('.fetchCallback()', () => {
				let response = {
					hash: null,
					status: 200,
					responseTime: 100,
					responseURL: '',
					data: '',
					class: ''
				};

				beforeEach(() => {
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker['syncWorker'].communication, 'sendDataMessage').and.stub();
					spyOn(worker['syncWorker'].endpoints, 'processXhrResponse').and.returnValue(response);
				});

				it('should call syncWorker.communication.sendDataMessage with right error parameters when data is null', () => {
					// Prepare
					response.data = null;

					// Execute
					worker['syncWorker'].endpoints.fetchCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('ENDPOINTS_UPDATE_ERROR', 'fake class', response);
				});

				it('should call syncWorker.communication.sendDataMessage with right parameters when data is not null', () => {
					// Prepare
					response.data = '';

					// Execute
					worker['syncWorker'].endpoints.fetchCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('ENDPOINTS_UPDATE_RESPONSE', 'fake class', response);
				});
			});

			describe('.fetchErrorCallback()', () => {
				it('should call syncWorker.communication.sendDataMessage with right parameters', () => {
					// Prepare
					const response = {
						status: 500,
						responseTime: 100,
						responseURL: '',
						data: ''
					};
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker['syncWorker'].communication, 'sendDataMessage').and.stub();
					spyOn(worker['syncWorker'].endpoints, 'processXhrErrorResponse').and.returnValue(response);

					// Execute
					worker['syncWorker'].endpoints.fetchErrorCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('ENDPOINTS_UPDATE_ERROR', 'fake class', response);
				});
			});

			describe('.snapshotUpdate()', () => {
				it('should call loadRemote with right parameters', () => {
					// Prepare
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker, 'loadRemote').and.stub();

					// Execute
					worker['syncWorker'].endpoints.snapshotUpdate({
						class: 'fake class',
						url: 'url-test',
						deviceType: 'fake device type'
					}, { extra: 'test' });

					// Expect
					expect(worker.loadRemote).toHaveBeenCalledWith(
						'fake class',
						'url-test',
						worker['syncWorker'].endpoints.snapshotUpdateCallback,
						worker['syncWorker'].endpoints.snapshotUpdateErrorCallback,
						{ extra: 'test' },
						'fake device type'
					);
				});
			});

			describe('.snapshotUpdateCallback()', () => {
				let response = {
					hash: null,
					status: 200,
					responseTime: 100,
					responseURL: '',
					data: '',
					class: ''
				};

				beforeEach(() => {
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker['syncWorker'].communication, 'sendDataMessage').and.stub();
					spyOn(worker['syncWorker'].endpoints, 'processXhrResponse').and.returnValue(response);
				});

				it('should call syncWorker.communication.sendDataMessage with right error parameters when data is null', () => {
					// Prepare
					response.data = null;

					// Execute
					worker['syncWorker'].endpoints.snapshotUpdateCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('SNAPSHOT_UPDATE_ERROR', 'fake class', response);
				});

				it('should call syncWorker.communication.sendDataMessage with right parameters when data is not null', () => {
					// Prepare
					response.data = '';

					// Execute
					worker['syncWorker'].endpoints.snapshotUpdateCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('SNAPSHOT_UPDATE_RESPONSE', 'fake class', response);
				});
			});

			describe('.snapshotUpdateErrorCallback()', () => {
				it('should call syncWorker.communication.sendDataMessage with right parameters', () => {
					// Prepare
					const response = {
						status: 500,
						responseTime: 100,
						responseURL: '',
						data: ''
					};
					spyOn(worker['syncWorker'].logger, 'log').and.stub();
					spyOn(worker['syncWorker'].communication, 'sendDataMessage').and.stub();
					spyOn(worker['syncWorker'].endpoints, 'processXhrErrorResponse').and.returnValue(response);

					// Execute
					worker['syncWorker'].endpoints.snapshotUpdateErrorCallback('fake class', {});

					// Expect
					expect(worker['syncWorker'].communication.sendDataMessage).toHaveBeenCalledWith('SNAPSHOT_UPDATE_ERROR', 'fake class', response);
				});
			});

			describe('.abortAll()', () => {
				it('should call abort method for each request', () => {
					// Prepare
					let request = jasmine.createSpyObj('request', ['abort']);
					let runningRequests = worker['runningRequests'];
					request.abort.and.stub();
					worker['runningRequests'] = [request, request, request];

					// Execute
					worker['syncWorker'].endpoints.abortAll();

					worker['runningRequests'] = runningRequests;

					// Expect
					expect((request.abort as jasmine.Spy).calls.count()).toBe(3);
				});
			});
		});
	});
});
