import { EventEmitter } from 'events';

const hashCode = (str): any => {
	let hash = 0, i, chr, len;
	if (str.length === 0) {
		return hash;
	}

	for (i = 0, len = str.length; i < len; i++) {
		chr = str.charCodeAt(i);
		hash = ((hash << 5) - hash) + chr;
		hash |= 0; // Convert to 32bit integer
	}

	return hash;
};

export default class IOSWorker extends EventEmitter {
	private httpTimeout;
	private listener;
	private runningRequests = {};

	private conf = {
		serverPrefix: 'http://localhost/tloms/',
		mainEndpointUrl: 'MyTime',
		log: true
	};

	constructor() {
		super();
	}

	sendMessage(message) {
		this.listener({
			data: message
		});
	}

	postMessage(msg) {
		const e = {
			data: msg
		};

		switch (e.data.message) {
			case 'ENDPOINTS_UPDATE':
				this.sendMessage({message: 'LOG', payload: 'this.syncWorker: Received ENDPOINTS_UPDATE request'});
				this.syncWorker.setLoggerLevel(e.data.payload);
				this.syncWorker.endpoints.setup(e.data.payload, e.data.metadata);
				break;

			case 'SNAPSHOT_UPDATE':
				this.sendMessage({message: 'LOG', payload: 'this.syncWorker: Received SNAPSHOT_UPDATE request'});
				this.syncWorker.setLoggerLevel(e.data.payload);
				this.syncWorker.endpoints.snapshotUpdate(e.data.payload, e.data.metadata);
				break;

			case 'REQUEST_CANCEL':
				this.sendMessage({message: 'LOG', payload: 'this.syncWorker: Received REQUEST_CANCEL request'});
				this.syncWorker.endpoints.abortAll();
				break;

			case 'SET_TIMEOUT':
				this.httpTimeout = e.data.timeout;
				break;

			default:
		}
	}

	addEventListener(message, fn) {
		this.listener = fn;
		(<any>this).addListener(message, fn);
	}

	terminate() {
		(<any>this).removeAllListeners();
	}

	addParameterToURL(strUrl, param, value) {
		param = param + '=' + value;
		strUrl += (strUrl.split('?')[1] ? '&' : '?') + param;

		return strUrl;
	}

	loadRemote(strClass, strUrl, successCallback, errorCallback, metadata, deviceType?) {
		this.syncWorker.logger.log(`loadRemote, class: ${strClass}, url: ${strUrl}`);
		let xmlHttpTimeout;
		let xhr;
		let headers = {};

		if (typeof XMLHttpRequest !== 'undefined') {
			xhr = new XMLHttpRequest();
		} else {
			let versions = ['MSXML2.XmlHttp.5.0',
				'MSXML2.XmlHttp.4.0',
				'MSXML2.XmlHttp.3.0',
				'MSXML2.XmlHttp.2.0',
				'Microsoft.XmlHttp'];

			for (let i = 0, len = versions.length; i < len; i++) {
				try {
					// xhr = new ActiveXObject(versions[i]);
					break;
				} catch (e) {
					console.error(e);
				}
			} // end for
		}

		const timeoutControl = new Date().getTime();
		xhr.onreadystatechange = ensureReadiness;
		xhr.withCredentials = true;

		function onNetworkError() {
			if (xhr.readyState < 4) {
				return;
			}
			xhr.isNoResponse = true;
		}

		xhr.onerror = onNetworkError;
		xhr.ontimeout = onNetworkError;

		xhr.onabort = () => {
			// don't do anything when it's aborted.
		};

		xhr.onloadend = () => {
			if (xhr.readyState < 4) {
				return;
			}

			xhr.responseTime = ((new Date().getTime()) - timeoutControl);

			// all is well
			if (xhr.readyState === 4) {
				clearTimeout(xmlHttpTimeout);
				if (xhr.canceled) {
					return;
				}
				if (xhr.status >= 200 && xhr.status < 400) {
					successCallback(strClass, xhr);
				} else {

					errorCallback(strClass, xhr);
				}
			}
		};

		function ensureReadiness() {
			if (xhr.readyState < 4) {
				return;
			}
		}

		xhr.failedURL = strUrl;
		// Add a timestamp to the url (cache avoid)
		const tstamp = new Date().getTime();
		const param = 'd';
		strUrl = this.addParameterToURL(strUrl, param, tstamp);

		xhr.uniqueToken = `tok_${tstamp}`;
		xhr.canceled = false;
		this.runningRequests[xhr.uniqueToken] = xhr;

		xhr.open('GET', strUrl, true);

		// Adds the 'Accept' field to the HTTP header for content negotiation
		xhr.setRequestHeader('Accept', this.formatContentNegotiation(metadata));
		// Adds the 'X-Platform' field to the HTTP header for device type
		xhr.setRequestHeader('X-PLATFORM', deviceType);

		xhr.send('');

		xmlHttpTimeout = setTimeout(ajaxTimeout, this.httpTimeout);

		function ajaxTimeout() {
			xhr.abort();
		}
	}

	/**
	 * Creates the content negotiation field value based on the data in metadata.
	 *
	 * @param  {String} metadata
	 * @return {String}
	 */
	formatContentNegotiation(metadata) {
		let ver, type, mime;

		ver = metadata.apiVer || 'v1';
		type = metadata.type || 'application/vnd.paychex.com';
		mime = metadata.mime || 'json';

		return mime + '+' + ver + '+' + type;
	}

	syncWorker = {
		logger: {
			log: (strLocation, strEvent?) => {
				if (strEvent === undefined) {
					strEvent = '';
				} else {
					strEvent = ' - ' + strEvent;
				}

				if (this.conf.log) {
					try {
						console.log('WebWorker - ' + strLocation + strEvent);
					} catch (err) {
						console.error(err);
					}
				}
			}
		},

		setLoggerLevel: (options) => {
			if (options !== null && options !== undefined && options.hasOwnProperty('devMode') && options.devMode) {
				this.conf.log = true;
			} else {
				this.conf.log = false;
			}
		},

		communication: {

			sendDataMessage: (strEvent, strClass, data) => {
				const message = {
					message: strEvent,
					payload: {
						class: strClass,
						data
					}
				};

				this.sendMessage(message);
			}
		},

		endpoints: {
			setup: (data, metadata) => {
				const strClass = data.class;
				const strUrl = data.url;

				this.syncWorker.logger.log('this.syncWorker->endpoints->setup');

				this.syncWorker.logger.log(`SETTING UP ENDPOINT URL: ${strUrl}`);
				this.syncWorker.endpoints.fetch(strClass, strUrl, metadata);
			},

			processXhrResponse: (xhr) => {
				const strResult = xhr.responseText;
				let payload = {
					hash: null,
					status: xhr.status,
					responseTime: xhr.responseTime,
					responseURL: xhr.responseURL,
					data: '',
					class: ''
				};
				try {
					const parsedData = JSON.parse(strResult);
					payload.data = parsedData;
					payload.hash = hashCode(strResult);
				} catch (err) {
					payload.data = null;
				}

				delete this.runningRequests[xhr.uniqueToken];

				return payload;
			},

			processXhrErrorResponse: (xhr) => {
				const strResult = xhr.responseText;

				delete this.runningRequests[xhr.uniqueToken];

				return {
					status: (xhr.isNoResponse) ? 0 : xhr.status,
					responseTime: xhr.responseTime,
					responseURL: xhr.responseURL ? xhr.responseURL : xhr.failedURL,
					data: strResult
				};
			},

			fetch: (strClass, strUrl, metadata) => {
				this.syncWorker.logger.log('syncWorker->endpoints->fetch', `class:  ${strClass}`);

				this.loadRemote(strClass, strUrl, this.syncWorker.endpoints.fetchCallback, this.syncWorker.endpoints.fetchErrorCallback, metadata);
			},

			fetchCallback: (strClass, xhr) => {
				this.syncWorker.logger.log('syncWorker->endpoints->fetchCallback', `class:  ${strClass}`);

				const payload = this.syncWorker.endpoints.processXhrResponse(xhr);
				payload.class = 'TloEndpoints';
				if (payload.data === null) {
					this.syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_ERROR', strClass, payload);
				} else {
					this.syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_RESPONSE', strClass, payload);
				}
			},

			fetchErrorCallback: (strClass, xhr) => {
				this.syncWorker.logger.log('this.syncWorker->endpoints->fetchErrorCallback',
					`class: ${strClass}, status: ${xhr.status}, responseText: ${xhr.responseText}`);

				const payload = this.syncWorker.endpoints.processXhrErrorResponse(xhr);
				this.syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_ERROR', strClass, payload);
			},

			snapshotUpdate: (data, metadata) => {
				this.syncWorker.logger.log('this.syncWorker->snapshotUpdate', `class: ${data.class}`);

				const strClass = data.class;
				const strUrl = data.url;
				this.loadRemote(strClass, strUrl, this.syncWorker.endpoints.snapshotUpdateCallback,
					this.syncWorker.endpoints.snapshotUpdateErrorCallback, metadata, data.deviceType);
			},

			snapshotUpdateCallback: (strClass, xhr) => {
				this.syncWorker.logger.log('this.syncWorker->endpoints->snapshotUpdateCallback', `class:  ${strClass}`);

				const payload = this.syncWorker.endpoints.processXhrResponse(xhr);
				if (payload.data === null) {
					this.syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_ERROR', strClass, payload);
				} else {
					this.syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_RESPONSE', strClass, payload);
				}
			},

			snapshotUpdateErrorCallback: (strClass, xhr) => {
				this.syncWorker.logger.log('this.syncWorker->endpoints->snapshotUpdateErrorCallback',
					`class: ${strClass}, status: ${xhr.status}, responseText: ${xhr.responseText}`);

				const payload = this.syncWorker.endpoints.processXhrErrorResponse(xhr);
				this.syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_ERROR', strClass, payload);
			},

			abortAll: () => {
				console.log('SyncWorker->Aborting all requests');
				for (let req in this.runningRequests) {
					if (this.runningRequests.hasOwnProperty(req)) {
						this.runningRequests[req].canceled = true;
						this.runningRequests[req].abort();
					}
				}
			}
		}
	};
}
